unit frameSynchDevice;

{$mode ObjFPC}{$H+}

interface

uses
  Classes, Dialogs, SysUtils, Forms, Controls, StdCtrls, ExtCtrls, RxDBGrid,
  memds, SQLDB,Graphics, SQLDBLib, SQLite3Conn, DB, Grids, DBGrids;

type

  { TframeSynchDevice }

  TframeSynchDevice = class(TFrame)
    dsDraw: TDataSource;
    dsmyDB: TDataSource;
    mdsDraw: TMemDataset;
    mdsmyDB: TMemDataset;
    PanelSynchConrol: TPanel;
    PanelSynchDraw: TPanel;
    PanelSynchBD: TPanel;
    panelSynchSplitter: TSplitter;
    gridDraw: TRxDBGrid;
    gridmyDB: TRxDBGrid;
    qryDraw: TSQLQuery;
    qrymyDB: TSQLQuery;
    connDB: TSQLite3Connection;
    libLoader: TSQLDBLibraryLoader;
    trDB: TSQLTransaction;
    procedure FrameResize(Sender: TObject);
    procedure GridDrawColumnCell(Sender: TObject;
  const Rect: TRect; DataCol: Integer; Column: TColumn;
  State: TGridDrawState);
    procedure panelSynchSplitterCanOffset(Sender: TObject; var NewOffset: Integer;
      var Accept: Boolean);
    procedure createDataSetA;
    procedure ConfigureGridDrawColumns;
    procedure createDataSetA2;
    procedure ConfigureGridmyDBColumns;
    procedure CompareAndFilterDatasets;
    procedure DoGridRefresh(Sender: TObject; var Done: Boolean);
  private
    FProportion: Double; // Пропорция ширины PanelSynch / (ClientWidth - Splitter)
    FIsResizingByUser: Boolean;
    procedure UpdateProportion;
  public
  procedure Activate;
  end;

implementation

{$R *.lfm}

{ TframeSynchDevice }
const

  A_selSQL = 'SELECT A, T1, T2, T3, T4 FROM A ORDER BY 1';
  A2_selSQL = 'SELECT A, T1, T2, T3, T4 FROM A2 ORDER BY 1';

procedure TframeSynchDevice.panelSynchSplitterCanOffset(Sender: TObject;
  var NewOffset: Integer; var Accept: Boolean);
begin
  if ClientWidth > panelSynchSplitter.Width then
    FProportion := PanelSynchDraw.Width / (ClientWidth - panelSynchSplitter.Width);
end;

procedure TframeSynchDevice.FrameResize(Sender: TObject);
begin
   if not FIsResizingByUser then
    PanelSynchDraw.Width := Round((ClientWidth - panelSynchSplitter.Width) * FProportion);
end;


procedure TframeSynchDevice.UpdateProportion;
begin
  if ClientWidth > panelSynchSplitter.Width then
    FProportion := PanelSynchDraw.Width / (ClientWidth - panelSynchSplitter.Width);
end;

procedure TframeSynchDevice.Activate;
begin
  // Здесь код, который должен выполняться при отображении фрейма
  //ShowMessage('Фрейм активирован');
  // Инициализация: показываем первый фрейм по умолчанию

  // первый контейнер (левая половина)
  PanelSynchDraw.Align := alLeft;
  PanelSynchDraw.Width := Self.ClientWidth div 2;
  PanelSynchDraw.BevelOuter := bvNone;
  PanelSynchDraw.BorderSpacing.Around := 2;

  // Настройка разделителя
  panelSynchSplitter.Align := alLeft;
  panelSynchSplitter.Width := 8;
  panelSynchSplitter.ResizeStyle := rsUpdate;
  panelSynchSplitter.MinSize := 100; // Минимальный размер панелей

  // второй контейнер (правая половина)
  PanelSynchBD.Align := alClient; // Займет оставшееся пространство
  PanelSynchBD.BevelOuter := bvNone;
  PanelSynchBD.BorderSpacing.Around := 2;

  Constraints.MinWidth := 600;
  Constraints.MinHeight := 400;

  FProportion := 0.5; // начальная пропорция

  //Работа с базой данных и гридам
    // 1. Настройка подключения к БД
  connDB.DatabaseName := 'SampleDEV.db3';
  connDB.CharSet := 'UTF8';
  connDB.Transaction:=trDB;

  // 2. Настройка MemDataset Draw
  with mdsDraw do
  begin
    FieldDefs.Add('A', ftInteger);
    FieldDefs.Add('T1', ftString, 50);
    FieldDefs.Add('T2', ftString, 50);
    FieldDefs.Add('T3', ftString, 50);
    FieldDefs.Add('T4', ftString, 50);
    FieldDefs.Add('DiffFlags', ftString, 255);
    CreateTable;

  end;
    // 2.1 Настройка MemDataset myDB
  with mdsmyDB do
  begin
    FieldDefs.Add('A', ftInteger);
    FieldDefs.Add('T1', ftString, 50);
    FieldDefs.Add('T2', ftString, 50);
    FieldDefs.Add('T3', ftString, 50);
    FieldDefs.Add('T4', ftString, 50);
    FieldDefs.Add('DiffFlags', ftString, 255);
    CreateTable;
  end;

   // 3. Связывание компонентов Draw
  dsDraw.DataSet := mdsDraw;
  gridDraw.DataSource := dsDraw;

   // 3.1 Связывание компонентов MyDB
  dsmyDB.DataSet := mdsmyDB;
  gridmyDB.DataSource := dsmyDB;

  // 4. Настройка запроса Draw
  qryDraw.Database := connDB;
  qryDraw.Transaction := trDB;

  // 4.1 Настройка запроса myDB
  qrymyDB.Database := connDB;
  qrymyDB.Transaction := trDB;

  try
    libLoader.ConnectionType := 'SQLite3';
    libLoader.LibraryName := 'sqlite3.dll';
    libLoader.Enabled := True;

    connDB.Connected := True;
    createDataSetA; // Сначала загружаем данные
    createDataSetA2; // Сначала загружаем данные

    CompareAndFilterDatasets;

    // Теперь настроим колонки грида
    ConfigureGridDrawColumns; // Явно создаем колонки для данных
    ConfigureGridmyDBColumns; // Явно создаем колонки для данных



    gridDraw.OnDrawColumnCell := @GridDrawColumnCell;
    gridmyDB.OnDrawColumnCell := @GridDrawColumnCell;

    // Принудительно перерисовываем
    gridDraw.Repaint;
    gridDraw.Invalidate;
    gridDraw.Update;

    gridmyDB.Repaint;
    gridmyDB.Invalidate;
    gridmyDB.Update;

    Application.OnIdle:=@DoGridRefresh;
  except
    on E: Exception do
      ShowMessage('Ошибка: ' + E.Message);
  end;

  // Добавьте в конец FormCreate:
  gridDraw.Align := alClient; // Грид займет все доступное пространство
  //gridDraw.Font.Color := clBlack;  // Основной цвет текста
  //gridDraw.TitleFont.Color := clNavy; // Цвет заголовков
  //gridDraw.FixedColor := clBtnFace; // Цвет фиксированных колонок
end;

procedure TframeSynchDevice.createDataSetA;
begin
  try
    mdsDraw.DisableControls;
    qryDraw.SQL.Text := A_selSQL;
    qryDraw.Open;

    mdsDraw.Close;
    mdsDraw.Open; // Теперь датасет пуст и готов к заполнению

    if not qryDraw.IsEmpty then
    begin
      qryDraw.First;
      while not qryDraw.EOF do
      begin
        mdsDraw.Append;
        mdsDraw.FieldByName('A').AsInteger := qryDraw.FieldByName('A').AsInteger;
        mdsDraw.FieldByName('T1').AsString := qryDraw.FieldByName('T1').AsString;
        mdsDraw.FieldByName('T2').AsString := qryDraw.FieldByName('T2').AsString;
        mdsDraw.FieldByName('T3').AsString := qryDraw.FieldByName('T3').AsString;
        mdsDraw.FieldByName('T4').AsString := qryDraw.FieldByName('T4').AsString;
        mdsDraw.Post;
        qryDraw.Next;
      end;
    end;
    qryDraw.Close;
  finally
    mdsDraw.EnableControls;
  end;
end;

procedure TframeSynchDevice.ConfigureGridDrawColumns;
var
  i: Integer;
begin
  gridDraw.Columns.BeginUpdate;
  try
    gridDraw.Columns.Clear;

    // Автоматическое создание колонок для всех полей
    for i := 0 to mdsDraw.FieldCount - 1 do
    begin
      with gridDraw.Columns.Add as TRxColumn do
      begin
        FieldName := mdsDraw.Fields[i].FieldName;
        Title.Caption := mdsDraw.Fields[i].DisplayLabel;
        if mdsDraw.Fields[i].DataType = ftString then
          Width := 100
        else
          Width := 70;

                // Установка цвета текста для колонки
        Font.Color := clBlack;
        Title.Font.Color := clNavy;

      end;
    end;
  finally
    gridDraw.Columns.EndUpdate;
  end;
end;

procedure TframeSynchDevice.createDataSetA2;
begin
  try
    mdsmyDB.DisableControls;
    qrymyDB.SQL.Text := A2_selSQL;
    qrymyDB.Open;

    // Вместо Clear закрываем и открываем датасет
    mdsmyDB.Close;
    mdsmyDB.Open; // Теперь датасет пуст и готов к заполнению

    if not qrymyDB.IsEmpty then
    begin
      qrymyDB.First;
      while not qrymyDB.EOF do
      begin
        mdsmyDB.Append;
        mdsmyDB.FieldByName('A').AsInteger := qrymyDB.FieldByName('A').AsInteger;
        mdsmyDB.FieldByName('T1').AsString := qrymyDB.FieldByName('T1').AsString;
        mdsmyDB.FieldByName('T2').AsString := qrymyDB.FieldByName('T2').AsString;
        mdsmyDB.FieldByName('T3').AsString := qrymyDB.FieldByName('T3').AsString;
        mdsmyDB.FieldByName('T4').AsString := qrymyDB.FieldByName('T4').AsString;
        mdsmyDB.Post;
        qrymyDB.Next;
      end;
    end;
    qrymyDB.Close;
  finally
    mdsmyDB.EnableControls;
  end;
end;

procedure TframeSynchDevice.ConfigureGridmyDBColumns;
var
  i: Integer;
begin
  gridmyDB.Columns.BeginUpdate;
  try
    gridmyDB.Columns.Clear;

    // Автоматическое создание колонок для всех полей
    for i := 0 to mdsmyDB.FieldCount - 1 do
    begin
      with gridmyDB.Columns.Add as TRxColumn do
      begin
        FieldName := mdsmyDB.Fields[i].FieldName;
        Title.Caption := mdsmyDB.Fields[i].DisplayLabel;
        if mdsmyDB.Fields[i].DataType = ftString then
          Width := 100
        else
          Width := 70;
      end;
    end;
  finally
    gridmyDB.Columns.EndUpdate;
  end;
end;

procedure TframeSynchDevice.CompareAndFilterDatasets;
var
  DiffFields: string;
  KeyValue: Variant;
  i: Integer;
  FieldName: string;
begin
  mdsDraw.DisableControls;
  mdsmyDB.DisableControls;
  try
    mdsDraw.First;
    while not mdsDraw.EOF do
    begin
      KeyValue := mdsDraw.FieldByName('A').AsInteger;
      DiffFields := '';
      if mdsmyDB.Locate('A', KeyValue, []) then
      begin
        for i := 0 to mdsDraw.FieldCount - 1 do
        begin
          FieldName := mdsDraw.Fields[i].FieldName;
          if (FieldName <> 'DiffFlags') and
             (mdsDraw.FieldByName(FieldName).AsString <> mdsmyDB.FieldByName(FieldName).AsString) then
            DiffFields := DiffFields + FieldName + ';';
        end;
      end
      else
        DiffFields := 'All'; // Если строка с таким A отсутствует — считаем всё разным

      mdsDraw.Edit;
      mdsDraw.FieldByName('DiffFlags').AsString := DiffFields;
      mdsDraw.Post;
      mdsDraw.Next;
    end;

    // Аналогично для второй таблицы
    mdsmyDB.First;
    while not mdsmyDB.EOF do
    begin
      KeyValue := mdsmyDB.FieldByName('A').AsInteger;
      DiffFields := '';
      if mdsDraw.Locate('A', KeyValue, []) then
      begin
        for i := 0 to mdsmyDB.FieldCount - 1 do
        begin
          FieldName := mdsmyDB.Fields[i].FieldName;
          if (FieldName <> 'DiffFlags') and
             (mdsmyDB.FieldByName(FieldName).AsString <> mdsDraw.FieldByName(FieldName).AsString) then
            DiffFields := DiffFields + FieldName + ';';
        end;
      end
      else
        DiffFields := 'All';

      mdsmyDB.Edit;
      mdsmyDB.FieldByName('DiffFlags').AsString := DiffFields;
      mdsmyDB.Post;
      mdsmyDB.Next;
    end;

    // Удалим все строки без различий
    mdsDraw.First;
    while not mdsDraw.EOF do
    begin
      if mdsDraw.FieldByName('DiffFlags').AsString = '' then
        mdsDraw.Delete
      else
        mdsDraw.Next;
    end;

    mdsmyDB.First;
    while not mdsmyDB.EOF do
    begin
      if mdsmyDB.FieldByName('DiffFlags').AsString = '' then
        mdsmyDB.Delete
      else
        mdsmyDB.Next;
    end;

  finally
    mdsDraw.EnableControls;
    mdsmyDB.EnableControls;
  end;
end;

procedure TframeSynchDevice.GridDrawColumnCell(Sender: TObject;
  const Rect: TRect; DataCol: Integer; Column: TColumn;
  State: TGridDrawState);
var
  Grid: TRxDBGrid;
  ds: TDataSet;
  DiffList: String;
  IsDifferent: Boolean;
  CellText: String;
begin
  Grid := Sender as TRxDBGrid;
  ds := Grid.DataSource.DataSet;

  if not Assigned(ds) or not ds.Active or ds.EOF then Exit;

  // Проверка на различие
  IsDifferent := False;
  if ds.FindField('DiffFlags') <> nil then
  begin
    DiffList := ds.FieldByName('DiffFlags').AsString;
    IsDifferent := Pos(Column.FieldName + ';', DiffList) > 0;
  end;

  CellText := Column.Field.AsString;

  with Grid.Canvas do
  begin
    // Цвета по умолчанию
    if gdSelected in State then
    begin
      Brush.Color := clHighlight;
      Font.Color := clHighlightText;
    end
    else if IsDifferent then
    begin
      Brush.Color := clRed;
      Font.Color := clWhite;
    end
    else
    begin
      Brush.Color := clWindow;
      Font.Color := clWindowText;
    end;

    // Заливка и текст
    FillRect(Rect);
    TextRect(Rect, Rect.Left + 2, Rect.Top + 2, CellText);
  end;
end;
procedure TframeSynchDevice.DoGridRefresh(Sender: TObject; var Done: Boolean);
begin
  gridDraw.Repaint;
  gridmyDB.Repaint;
      gridDraw.Invalidate;  // Перерисовать все строки
    gridmyDB.Invalidate;
  // Важно: сбрасываем обработчик, чтобы не вызывался постоянно
  Application.OnIdle := nil;
  Done := True; // выполняем один раз
end;



end.

